<?php

declare(strict_types=1);

use function apexl\Io\env;

return [
    'enable' => env('WHOOPS_ENABLE', false),
    'editor' => env('WHOOPS_EDITOR', 'io_phpstorm'),
    'local_base_path' => env('WHOOPS_LOCAL_BASE_PATH', ''),
];